
using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading;
using ClarionSimulation;
using WorldServerLibrary;
using WorldServerLibrary.Model;
using WorldServerLibrary.Exceptions;

namespace ClarionDEMO
{
	class MainClass
	{
		#region properties
		private WorldServer worldServer = null;
        private ClarionAgent agent;
        String creatureId = String.Empty;
        String creatureName = String.Empty;
		double prad = 0;
		#endregion

		#region constructor
		public MainClass() {
			Console.WriteLine ("Clarion Demo V0.4");
			agent = new ClarionAgent();
            agent.OnNewVisualSensorialInformation += new InputVisualSensorialInformationEventHandler(agent_OnNewVisualSensorialInformation);
            agent.OnNewExternalActionSelected += new OutputActionChunkEventHandler(agent_OnNewExternalActionSelected);
			try
            {
                worldServer = new WorldServer("localhost", 4011);
                String message = worldServer.Connect();

                if (worldServer != null && worldServer.IsConnected)
                {
                    Console.Out.WriteLine ("[SUCCESS] " + message + "\n");
                    worldServer.NewCreature(400, 200, 0, out creatureId, out creatureName);
					worldServer.NewBrick(4, 747, 2, 800, 567);
                    worldServer.NewBrick(4, 50, -4, 747, 47);
                    worldServer.NewBrick(4, 49, 562, 796, 599);
                    worldServer.NewBrick(4, -2, 6, 50, 599);

					worldServer.GrowWorld();
					Thread.Sleep(5000);

					worldServer.SendGenLeaflet();


                    if (!String.IsNullOrWhiteSpace(creatureId))
                    {
                        worldServer.SendStartCamera(creatureId);
                        worldServer.SendStartCreature(creatureId);
                    }

					Console.Out.WriteLine("Creature created with name: " + creatureId + "\n");
                    agent.Run();
					Console.Out.WriteLine("Running Simulation ...\n");
                }
            }
            catch (WorldServerInvalidArgument invalidArtgument)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Invalid Argument: {0}\n", invalidArtgument.Message));
            }
            catch (WorldServerConnectionError serverError)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Is is not possible to connect to server: {0}\n", serverError.Message));
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Unknown Error: {0}\n", ex.Message));
            }
		}
		#endregion

		#region Methods
		public static void Main (string[] args)	{
			new MainClass();
		}

        IList<Thing> agent_OnNewVisualSensorialInformation()
        {
            IList<Thing> response = null;

            if (worldServer != null && worldServer.IsConnected)
            {
                response = worldServer.SendGetCreatureState(creatureName);
				prad = (Math.PI / 180) * response.First().Pitch;
			    while (prad > Math.PI) prad -= 2 * Math.PI;
			    while (prad < - Math.PI) prad += 2 * Math.PI;

            }


            return response;
        }

        void agent_OnNewExternalActionSelected(ClarionAgentActionType externalAction)
        {   Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
            if (worldServer != null && worldServer.IsConnected)
            {
                switch (externalAction)
                {
				case ClarionAgentActionType.DO_NOTHING:
						worldServer.SendSetGoTo(creatureId, 0,0,0,0);
						worldServer.SendStopCreature (creatureId);
                        break;
                   case ClarionAgentActionType.ROTATE_CLOCKWISE:
					    double newprad = prad + Math.PI/1000;
					    while (newprad > Math.PI) newprad -= 2 * Math.PI;
			            while (newprad < - Math.PI) newprad += 2 * Math.PI;
                        worldServer.SendSetAngle(creatureId, 0, 0, newprad);
                        break;
				case ClarionAgentActionType.SACK_JEWEL:
					Thing sackJewel = agent.CurrentSensorialInfo.Where (item => (item.CategoryId == Thing.CATEGORY_JEWEL)).OrderBy (item => item.DistanceToCreature).First ();
					worldServer.SendSackIt (creatureId, sackJewel.Name);
					bool already = false;
					foreach (Thing tk in agent.knapsack) {
						if (tk.Name.CompareTo (sackJewel.Name) == 0) {
							already = true;
							break;
						}
					}
					if (!already)
						agent.knapsack.Add (sackJewel);
					
					break;

				case ClarionAgentActionType.EAT_FOOD:
					Thing food = agent.CurrentSensorialInfo.Where (item => (item.CategoryId == Thing.CATEGORY_NPFOOD || item.CategoryId == Thing.categoryPFOOD)).OrderBy (item => item.DistanceToCreature).First ();
					worldServer.SendEatIt(creatureId, food.Name);
					break;

				case ClarionAgentActionType.GO_TO_JEWEL:
					Thing closestJewel = null;

					Thing closestLeafletJewel = null;

					Creature c = agent.CurrentSensorialInfo.First (item => (item.CategoryId == Thing.CATEGORY_CREATURE)) as Creature;

					if (agent.CurrentSensorialInfo != null) {
						foreach (Thing t in agent.CurrentSensorialInfo) {
							if (t.CategoryId == Thing.CATEGORY_JEWEL) {
								if (closestJewel != null && closestJewel.DistanceToCreature > t.DistanceToCreature) {
									closestJewel = t;
								} else if (closestJewel == null) {
									closestJewel = t;
								}

								if (closestLeafletJewel == null && c.IsLeafletMissingJewel (t.Material.Color)) {
									closestLeafletJewel = t;
								} else if (closestLeafletJewel != null && c.IsLeafletMissingJewel (t.Material.Color)) {
									if (closestLeafletJewel.DistanceToCreature > t.DistanceToCreature) {
										closestLeafletJewel = t;
									}
								}
							}
						}


						if (closestLeafletJewel != null) {
							closestJewel = closestLeafletJewel;
						}

						double x = (closestJewel.X1 + closestJewel.X2) / 2.0f;
						double y = (closestJewel.Y1 + closestJewel.Y2) / 2.0f;
						worldServer.SendSetGoTo(creatureId, 1.5f, 1.5f, x, y);
					}
						
                        break;
                    default:
                        break;
                }
            }
        }

        #endregion
	}
	
	
}
