﻿using System;
using System.Collections.Generic;

namespace WorldServerLibrary
{
	public class Leaflet
	{
		private Int64 ID;

		private int payment = 0; //number of points is gained by a creature when it is delivered

		//Type (i.e. color name), (Total number, Collected number)
		private Dictionary<String, int[]> itemsMap = new Dictionary<String, int[]>();

		public Leaflet(Int64 ID, Dictionary<String, int[]> items, int payment) {
			this.ID = ID;
			setItems(items);
			this.payment = payment;
		}


		public Dictionary<String, int[]> getItems() {

			return itemsMap;
		}

		public void setItems(Dictionary<String, int[]> items){
			//itemsMap.Clear ();
			foreach (string str in items.Keys) {
				int[] values = items [str];

				itemsMap.Add (str, values);
			}
		}

		public int getPayment(){
			return payment;
		}

		public int getTotalNumberOfType(String type){

			if(itemsMap.ContainsKey(type)){
				int[] values = null;
				values = itemsMap[type];
				return values[0];
			}else return -1;
		}
		public int getCollectedNumberOfType(String type){

			if(itemsMap.ContainsKey(type)){
				int []values = null;
				values = itemsMap[type];
				return values[1];
			}else return -1;
		}

		public int getMissingNumberOfType(String type){

			if(itemsMap.ContainsKey(type)){
				int []values = null;
				values = itemsMap[type];
				return (values[0] - values[1]);
			}else return -1;
		}

		public Dictionary<String, int> getWhatToCollect(){

			Dictionary<String, int> itemsToSearch = new Dictionary<String, int>();
			foreach (string str in itemsMap.Keys) {
				itemsToSearch.Add(str, getMissingNumberOfType(str));
			}
			return itemsToSearch;
		}

		public int getTotalMissing(){

			int total = 0;
			foreach (string str in itemsMap.Keys) {
				int miss =  getMissingNumberOfType(str);
				if (miss > 0) {
					total += miss;
				}
			}
			return total;
		}

		public bool ifInLeaflet(String type) {

			if (itemsMap.ContainsKey(type)) return true;
			else return false;
		}

		public Int64 getID() {
			return ID;
		}

		public String toString(){
			String ret = "LeafletID: "+this.ID+" ";
			foreach (string str in itemsMap.Keys) {
				ret = ret + "Type: "+ str + " ";
				//(total number) (collected)
				ret = ret + itemsMap[str][0] + " ";
				ret = ret + itemsMap[str][1] + " ";
			}
			//ret = ret+" payment= "+payment+" ";
			return ret;
		}
			
	}
}

